/*
 * Bluemote - use your Bluetooth enabled T610 or compatible Sony Ericsson phone as
 * a remote for your PC.
 * Copyright (C) 2004
 * Saravana K Kannan <saravkrish at uky dot edu>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/stat.h>
#include <string.h>
#include "bluemote.h"
#include "ipc.h"

int fdlock;
int fdfifo=-1;
char lockfile[MAXLEN];
char ipcfile[MAXLEN];
struct sockaddr_un sockadr;
int sock;

int get_lock()
{
 struct flock fl;

 strcpy(lockfile, getenv("HOME"));
 strcat(lockfile, BLUEMOTEDIR);
 strcat(lockfile, LOCKFILE);

 fl.l_type = F_WRLCK;
 fl.l_whence = SEEK_SET;
 fl.l_start = 0;
 fl.l_len = 1;

 if((fdlock = open(lockfile, O_WRONLY|O_CREAT, 0600)) == -1)
  return 0;

 if(fcntl(fdlock, F_SETLK, &fl) == -1)
  return 0;

 return 1;
}

int open_consumer()
{
 int len;
 
 strcpy(ipcfile, getenv("HOME"));
 strcat(ipcfile, BLUEMOTEDIR);
 strcat(ipcfile, IPCFILE);

 if((sock=socket(AF_UNIX, SOCK_DGRAM, 0))==-1)
  return 0;
 if(fcntl(sock, F_SETFL, O_NONBLOCK)==-1)
 {
  close(sock);
  return(0);
 }
 
 sockadr.sun_family = AF_UNIX;
 strcpy(sockadr.sun_path, ipcfile);
 unlink(sockadr.sun_path);

 len = strlen(sockadr.sun_path) + sizeof(sockadr.sun_family);
 if(bind(sock, (struct sockaddr *) &sockadr, len)==-1)
 {
  close(sock);
  return 0;
 }
 
 return 1;
}
 
int open_producer()
{
 int len;
 
 strcpy(ipcfile, getenv("HOME"));
 strcat(ipcfile, BLUEMOTEDIR);
 strcat(ipcfile, IPCFILE);

 if((sock=socket(AF_UNIX, SOCK_DGRAM, 0))==-1)
  return 0;
 
 sockadr.sun_family = AF_UNIX;
 strcpy(sockadr.sun_path, ipcfile);

 len = strlen(sockadr.sun_path) + sizeof(sockadr.sun_family);
 if(connect(sock, (struct sockaddr *) &sockadr, len)==-1)
 {
  close(sock);
  return 0;
 }
 
 return 1;
}

void close_consumer()
{
 close(sock);
 unlink(ipcfile);
}

void close_producer()
{
 close(sock);
}

int ipc_read(char *buf, int len)
{
 int cnt;

 cnt = recv(sock, buf, len, 0);

 if(cnt==-1) return -errno;
 if(!cnt) return 0;

 buf[cnt]=0;
 logger("MSG",buf);
 return cnt;
}

int ipc_write(char *buf, int len)
{
 int cnt;

 cnt = send(sock, buf, len, 0);

 return cnt;
}
